<?xml version="1.0"?><api><query><pages><page pageid="27697087" ns="0" title="API"><revisions><rev revid="367741756" parentid="367741564" minor="" user="Graham87" timestamp="2010-06-13T08:41:17Z" size="70" comment="Protected API: restore protection ([edit=sysop] (indefinite) [move=sysop] (indefinite))" xml:space="preserve">#REDIRECT [[Application programming interface]]{{R from abbreviation}}</rev></revisions></page></pages></query></api>
