<?xml version="1.0"?><api><query><pages><page pageid="27697087" ns="0" title="API"><revisions><rev user="Graham87" /></revisions></page><page pageid="15580374" ns="0" title="Main Page"><revisions><rev user="Rich Farmbrough" /></revisions></page></pages></query></api>
