<?xml version="1.0"?>
<api>
<query>
<pages>
<page pageid="27697087" ns="0" title="API">
<revisions>
<rev user="martine" timestamp="2010-06-13T08:41:17Z" comment="Protected API: restore protection ([edit=sysop] (indefinite) [move=sysop] (indefinite))" parsetree="&lt;root&gt;#REDIRECT [[Application programming interface]]&lt;template&gt;&lt;title&gt;R from abbreviation&lt;/title&gt;&lt;/template&gt;&lt;/root&gt;" xml:space="preserve">#REDIRECT [[Application programming interface]]{{R from abbreviation}}</rev>
</revisions>
</page>
<page pageid="27697087" ns="0" title="API">
<revisions>
    <rev user="Graham87" timestamp="2010-06-13T08:41:17Z" comment="Protected API: restore protection ([edit=sysop] (indefinite) [move=sysop] (indefinite))" parsetree="&lt;root&gt;#REDIRECT [[Application programming interface]]&lt;template&gt;&lt;title&gt;R from abbreviation&lt;/title&gt;&lt;/template&gt;&lt;/root&gt;" xml:space="preserve">#REDIRECT [[Application programming interface]]{{R from abbreviation}}</rev>
</revisions>
</page>
</pages>
</query>
</api>
